#pragma once

#include <windows.h>

#include <map>
#include <istream>
#include <ostream>

class IOverlay
{
public:
	virtual void operator()(LPVOID pDest, LPCVOID pSource, SIZE_T bytes) const = 0;

public:
	virtual const char* typeCode() const = 0;
	virtual bool isOpaque() const = 0;
};

typedef const IOverlay* HOverlay;

class OpaqueOverlay : public IOverlay
{
public:
	void operator()(LPVOID pDest, LPCVOID pSource, SIZE_T bytes) const;
public:
	const char* typeCode() const;
	bool isOpaque() const;
};

template <typename ImplT>
class Overlay : public IOverlay
	// where
	// RGBTRIPLE source, target;
	// target = ImplT()(target, source);
{
protected:
	typedef Overlay Base;
public:
	void operator()(LPVOID pDest, LPCVOID pSource, SIZE_T bytes) const;
	bool isOpaque() const;
};

class ChromaOverlay : public Overlay<ChromaOverlay>
{
public:
	using Base::operator();
	RGBTRIPLE operator()(RGBTRIPLE target, RGBTRIPLE source) const;
public:
	const char* typeCode() const;
};

class LumaOverlay : public Overlay<LumaOverlay>
{
public:
	using Base::operator();
	RGBTRIPLE operator()(RGBTRIPLE target, RGBTRIPLE source) const;
public:
	const char* typeCode() const;
};

class Overlays
{
private:
	std::map<std::string, HOverlay> m_overlays;
public:
	static Overlays& instance();
	HOverlay operator[](const std::string& name) const;
	void insert(const char* name, HOverlay overlay);

	template <typename OverlayT>
	class Register : public OverlayT
	{
	public:
		Register(const char* name);
	};
};

std::istream& operator>>(std::istream& stream, HOverlay& pOverlay);
std::ostream& operator<<(std::ostream& stream, HOverlay pOverlay);

#include "OverlayMode.inline.h"
