#pragma once

#include <algorithm>

inline void OpaqueOverlay::operator()(LPVOID pDest, LPCVOID pSource, SIZE_T bytes) const
{
	memcpy(pDest, pSource, bytes);
}

inline const char* OpaqueOverlay::typeCode() const
{
	return "Blend";
}

inline bool OpaqueOverlay::isOpaque() const
{
	return true;
}

template <typename ImplT>
void Overlay<ImplT>::operator()(LPVOID pDest, LPCVOID pSource, SIZE_T bytes) const
{
	std::transform(
		reinterpret_cast<const RGBTRIPLE*>(pDest), reinterpret_cast<const RGBTRIPLE*>(pDest) + bytes / sizeof(RGBTRIPLE),
		reinterpret_cast<const RGBTRIPLE*>(pSource),
		reinterpret_cast<RGBTRIPLE*>(pDest),
		static_cast<const ImplT&>(*this));
}

template <typename ImplT>
bool Overlay<ImplT>::isOpaque() const
{
	return false;
}


inline BYTE GetY(BYTE R, BYTE G, BYTE B)
{
	return (( 66 * R + 129 * G +  25 * B + 128) >> 8) +  16;
}

inline BYTE GetU(BYTE R, BYTE G, BYTE B)
{
	return ((-38 * R -  74 * G + 112 * B + 128) >> 8) + 128;
}

inline BYTE GetV(BYTE R, BYTE G, BYTE B)
{
	return ((112 * R -  94 * G -  18 * B + 128) >> 8) + 128;
}

inline BYTE GetY(RGBTRIPLE rgb)
{
	return GetY(rgb.rgbtRed, rgb.rgbtGreen, rgb.rgbtBlue);
}

inline BYTE GetU(RGBTRIPLE rgb)
{
	return GetU(rgb.rgbtRed, rgb.rgbtGreen, rgb.rgbtBlue);
}

inline BYTE GetV(RGBTRIPLE rgb)
{
	return GetV(rgb.rgbtRed, rgb.rgbtGreen, rgb.rgbtBlue);
}

inline BYTE clip(LONG v)
{
	return BYTE((std::min)(255L, (std::max)(0L, v)));
}

inline RGBTRIPLE YUV(BYTE Y, BYTE U, BYTE V)
{
	LONG C = Y - 16, D = U - 128, E = V - 128;
	RGBTRIPLE result =
	{
		clip((298 * C + 516 * D           + 128) >> 8),
		clip((298 * C - 100 * D - 208 * E + 128) >> 8),
		clip((298 * C           + 409 * E + 128) >> 8),
	};
	return result;
}


inline RGBTRIPLE ChromaOverlay::operator()(RGBTRIPLE target, RGBTRIPLE source) const
{
	return YUV(GetY(target), GetU(source), GetV(source));
}

inline const char* ChromaOverlay::typeCode() const
{
	return "Chroma";
}


inline RGBTRIPLE LumaOverlay::operator()(RGBTRIPLE target, RGBTRIPLE source) const
{
	return YUV(GetY(source), GetU(target), GetV(target));
}

inline const char* LumaOverlay::typeCode() const
{
	return "Luma";
}

template <typename OverlayT>
Overlays::Register<OverlayT>::Register(const char* name)
{
	Overlays::instance().insert(name, this);
}
