#include "stdafx.h"

#include "ViewMode.h"

#include "vclipDoc.h"

CRect ViewMode::viewFromFrame(const CRect& rect) const
{
	return CRect(viewFromFrame(rect.TopLeft()), viewFromFrame(rect.BottomRight()));
}

CRect ViewMode::frameFromView(const CRect& rect) const
{
	return CRect(frameFromView(rect.TopLeft()), frameFromView(rect.BottomRight()));
}


CPoint FrameMode::viewFromFrame(const CPoint& point) const
{
	return point;
}
CPoint FrameMode::frameFromView(const CPoint& point) const
{
	return point;
}

LONG FrameMode::next(LONG& frame, LONG& /*order*/) const
{
	return ++frame;
}
LONG FrameMode::previous(LONG& frame, LONG& /*order*/) const
{
	return --frame;
}
bool FrameMode::hasNext(LONG frame, LONG /*order*/, LONG frames) const
{
	return frame + 1 < frames;
}
bool FrameMode::hasPrevious(LONG frame, LONG /*order*/) const
{
	return frame > 0;
}
CString FrameMode::formatPosition(LONG frame, LONG /*order*/, LONG frames) const
{
	CString result;
	result.Format("Frame %d of %d", frame, frames);
	return result;
}

bool FrameMode::visible(LONG /*currentOrder*/, LONG /*targetOrder*/) const
{
	return true;
}

void FrameMode::draw(CDC* pDC, const DrawDib& drawDib, const PFrame& frame, LONG /*order*/) const
{
	drawDib.draw(
		pDC,   frame.rect(),
		frame, frame.rect(),
		0);
}

PFrameOperation FrameMode::paste(CVclipDoc& document, const Clipboard& clipboard, LONG frame, LONG order, HOverlay mode) const
{
	return document.add(
		the<FrameOperationFactory>().createFramePatch(document,
			clipboard.frame,
				clipboard.selection.left, clipboard.selection.top,
				clipboard.selection.Width(), clipboard.selection.Height(),
			frame, 
				clipboard.selection.left, clipboard.selection.top,
				clipboard.selection.Width(), clipboard.selection.Height(),
			mode));
}

FrameMode frameMode;


CPoint FieldMode::viewFromFrame(const CPoint& point) const
{
	return CPoint(point.x, (point.y - (point.y % 2 ? 1 : 0)) / 2);
}
CPoint FieldMode::frameFromView(const CPoint& point) const
{
	return CPoint(point.x, point.y * 2);
}

LONG FieldMode::next(LONG& frame, LONG& order) const
{
	switch (order)
	{
	case 0:
		order = 1;
		return frame;
	case 1:
		order = 0;
		return ++frame;
	}
	return frame;
}
LONG FieldMode::previous(LONG& frame, LONG& order) const
{
	switch (order)
	{
	case 1:
		order = 0;
		return frame;
	case 0:
		order = 1;
		return --frame;
	}
	return frame;
}
bool FieldMode::hasNext(LONG frame, LONG order, LONG frames) const
{
	return frame + order < frames;
}
bool FieldMode::hasPrevious(LONG frame, LONG order) const
{
	return frame + order > 0;
}
CString FieldMode::formatPosition(LONG frame, LONG order, LONG frames) const
{
	CString result;
	result.Format("Frame %d%c of %d", frame, order["bt"], frames);
	return result;
}

bool FieldMode::visible(LONG currentOrder, LONG targetOrder) const
{
	return currentOrder == targetOrder;
}

void FieldMode::draw(CDC* pDC, const DrawDib& drawDib, const PFrame& frame, LONG order) const
{
	PFrame fields(frame, SeparateFields());
	CRect fieldRect(0, 0, frame.width(), fields.height());
	drawDib.draw(
		pDC,    fieldRect,
		fields, fieldRect + fields.fieldOrigin(order),
		0);
}

PFrameOperation FieldMode::paste(CVclipDoc& document, const Clipboard& clipboard, LONG frame, LONG order, HOverlay mode) const
{
	return document.add(
		the<FrameOperationFactory>().createFieldPatch(document,
			clipboard.frame, clipboard.order,
				clipboard.selection.left, clipboard.selection.top,
				clipboard.selection.Width(), clipboard.selection.Height(),
			frame, order,
				clipboard.selection.left, clipboard.selection.top, 
				clipboard.selection.Width(), clipboard.selection.Height(),
			mode));
}

FieldMode fieldMode;


void Clipboard::copy(const ViewMode* aMode, LONG aFrame, LONG anOrder, const CRect& aSelection)
{
	mode = aMode;
	frame = aFrame;
	order = anOrder;
	selection = aSelection;
}

PFrameOperation Clipboard::paste(CVclipDoc& document, const ViewMode* aMode, 
																 LONG aFrame, LONG anOrder, HOverlay overlayMode) const
{
	return mode->paste(document, *this, aFrame, anOrder, overlayMode);
}

bool Clipboard::empty() const
{
	return !!selection.IsRectEmpty();
}

