#ifndef YK_ALGORITHMS_INCLUDED
#define YK_ALGORITHMS_INCLUDED

#include <algorithm>

namespace yk
{

template <typename FwdIter, typename PredicateF, typename FunctorF>
FunctorF for_each_if(FwdIter begin, FwdIter end, PredicateF isGood, FunctorF actOn)
{
	for (; begin != end; ++begin)
	{
		if (isGood(*begin)) actOn(begin);
	}
	return actOn;
};

template <typename FwdIter, typename PredicateF, typename LessF>
FwdIter min_element_if(FwdIter begin, FwdIter end, PredicateF isGood, LessF less)
{
	begin = std::find_if(begin, end, isGood);
	if (begin == end) return end;

	FwdIter result = begin++;

	for (; begin != end; ++begin)
	{
		if (isGood(*begin) && less(*begin, *result))
		{
			result = begin;
		}
	}
	return result;
}

template <typename FwdIter, typename PredicateF>
FwdIter min_element_if(FwdIter begin, FwdIter end, PredicateF isGood)
{
	begin = std::find_if(begin, end, isGood);
	if (begin == end) return end;

	FwdIter result = begin++;

	for (; begin != end; ++begin)
	{
		if (isGood(*begin) && *begin < *result)
		{
			result = begin;
		}
	}
	return result;
}

template <typename FwdIter, typename PredicateF, typename LessF>
FwdIter max_element_if(FwdIter begin, FwdIter end, PredicateF isGood, LessF less)
{
	begin = std::find_if(begin, end, isGood);
	if (begin == end) return end;

	FwdIter result = begin++;

	for (; begin != end; ++begin)
	{
		if (isGood(*begin) && less(*result, *begin))
		{
			result = begin;
		}
	}
	return result;
}

template <typename FwdIter, typename PredicateF>
FwdIter max_element_if(FwdIter begin, FwdIter end, PredicateF isGood)
{
	begin = std::find_if(begin, end, isGood);
	if (begin == end) return end;

	FwdIter result = begin++;

	for (; begin != end; ++begin)
	{
		if (isGood(*begin) && *result < *begin)
		{
			result = begin;
		}
	}
	return result;
}

}

#endif
