#ifndef YK_COMPOSE_H
#define YK_COMPOSE_H

#include <functional>
#include "type_traits.h"

namespace yk
{
	template <typename Fx, typename Gxy>
	class ComposeFGxy : public std::binary_function<
		typename Gxy::first_argument_type,
		typename Gxy::second_argument_type,
		typename Fx::result_type>
	{
	public:
		ComposeFGxy(const Fx& f, const Gxy& g)
		: f_(f), g_(g)
		{
		}

		typename Fx::result_type
		operator()(typename const_ref<typename Gxy::first_argument_type>::type x,
			typename const_ref<typename Gxy::second_argument_type>::type y) const
		{
			return f_(g_(x, y));
		}
	private:
		Fx f_;
		Gxy g_;
	};

	template <typename Fx, typename Gxy>
	ComposeFGxy<Fx, Gxy>
	compose_f_gxy(const Fx& f, const Gxy& g)
	{
		return ComposeFGxy<Fx, Gxy>(f, g);
	}

	template <typename Fxy, typename Gx, typename Hx = Gx>
	class ComposeFGxHy : public std::binary_function<
		typename Gx::argument_type,
		typename Hx::argument_type,
		typename Fxy::result_type>
	{
	public:
		ComposeFGxHy(const Fxy& f, const Gx& g, const Hx& h)
		: f_(f), g_(g), h_(h)
		{
		}

		typename Fxy::result_type
		operator()(typename const_ref<typename Gx::argument_type>::type x,
			typename const_ref<typename Hx::argument_type>::type y) const
		{
			return f_(g_(x), h_(y));
		}
	private:
		Fxy f_;
		Gx g_;
		Hx h_;
	};

	template <typename Fxy, typename Gx, typename Hx>
	ComposeFGxHy<Fxy, Gx, Hx>
	compose_f_gx_hy(const Fxy& f, const Gx& g, const Hx& h)
	{
		return ComposeFGxHy<Fxy, Gx, Hx>(f, g, h);
	}

	template <typename Fxy, typename Gx>
	ComposeFGxHy<Fxy, Gx, Gx>
	compose_f_gx_gy(const Fxy& f, const Gx& g)
	{
		return ComposeFGxHy<Fxy, Gx, Gx>(f, g, g);
	}
}

#endif
